@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 
 
 #include "libxfs_priv.h"
@@ -23,8 +24,22 @@ kmem_zone_init(int size, char *name)
 	return ptr;
 }
 
+int
+kmem_zone_destroy(kmem_zone_t *zone)
+{
+	int	leaked = 0;
+
+	if (getenv("LIBXFS_LEAK_CHECK") && zone->allocated) {
+		leaked = 1;
+		fprintf(stderr, "zone %s freed with %d items allocated\n",
+				zone->zone_name, zone->allocated);
+	}
+	free(zone);
+	return leaked;
+}
+
 void *
-kmem_zone_alloc(kmem_zone_t *zone, int flags)
+kmem_cache_alloc(kmem_zone_t *zone, gfp_t flags)
 {
 	void	*ptr = malloc(zone->zone_unitsize);
 
@@ -37,16 +52,16 @@ kmem_zone_alloc(kmem_zone_t *zone, int flags)
 	zone->allocated++;
 	return ptr;
 }
+
 void *
-kmem_zone_zalloc(kmem_zone_t *zone, int flags)
+kmem_cache_zalloc(kmem_zone_t *zone, gfp_t flags)
 {
-	void	*ptr = kmem_zone_alloc(zone, flags);
+	void	*ptr = kmem_cache_alloc(zone, flags);
 
 	memset(ptr, 0, zone->zone_unitsize);
 	return ptr;
 }
 
-
 void *
 kmem_alloc(size_t size, int flags)
 {
@@ -60,6 +75,12 @@ kmem_alloc(size_t size, int flags)
 	return ptr;
 }
 
+void *
+kmem_alloc_large(size_t size, int flags)
+{
+	return kmem_alloc(size, flags);
+}
+
 void *
 kmem_zalloc(size_t size, int flags)
 {
@@ -70,7 +91,7 @@ kmem_zalloc(size_t size, int flags)
 }
 
 void *
-kmem_realloc(void *ptr, size_t new_size, size_t old_size, int flags)
+kmem_realloc(void *ptr, size_t new_size, int flags)
 {
 	ptr = realloc(ptr, new_size);
 	if (ptr == NULL) {
