@@ -141,7 +141,7 @@ xfs_trans_dup(
 {
 	struct xfs_trans	*ntp;
 
-	ntp = kmem_zone_zalloc(xfs_trans_zone, 0);
+	ntp = kmem_cache_zalloc(xfs_trans_zone, 0);
 
 	/*
 	 * Initialize the new transaction structure.
@@ -258,7 +258,7 @@ libxfs_trans_alloc(
 	struct xfs_trans	*tp;
 	int			error;
 
-	tp = kmem_zone_zalloc(xfs_trans_zone, 0);
+	tp = kmem_cache_zalloc(xfs_trans_zone, 0);
 	tp->t_mountp = mp;
 	INIT_LIST_HEAD(&tp->t_items);
 	INIT_LIST_HEAD(&tp->t_dfops);
@@ -780,7 +780,11 @@ xfs_inode_item_put(
 {
 	struct xfs_inode		*ip = iip->ili_inode;
 
+	ASSERT(iip->ili_item.li_buf == NULL);
+
 	ip->i_itemp = NULL;
+
+	list_del_init(&iip->ili_item.li_bio_list);
 	kmem_cache_free(xfs_ili_zone, iip);
 }
 
@@ -793,49 +797,37 @@ xfs_inode_item_put(
  */
 static void
 inode_item_done(
-	xfs_inode_log_item_t	*iip)
+	struct xfs_inode_log_item	*iip)
 {
-	xfs_dinode_t		*dip;
-	xfs_inode_t		*ip;
-	xfs_mount_t		*mp;
-	xfs_buf_t		*bp;
-	int			error;
+	xfs_buf_t			*bp;
+	int				error;
 
-	ip = iip->ili_inode;
-	mp = iip->ili_item.li_mountp;
-	ASSERT(ip != NULL);
+	ASSERT(iip->ili_inode != NULL);
 
 	if (!(iip->ili_fields & XFS_ILOG_ALL))
-		goto free;
+		goto free_item;
 
-	/*
-	 * Get the buffer containing the on-disk inode.
-	 */
-	error = xfs_imap_to_bp(mp, NULL, &ip->i_imap, &dip, &bp, 0, 0);
-	if (error) {
-		fprintf(stderr, _("%s: warning - imap_to_bp failed (%d)\n"),
-			progname, error);
-		goto free;
-	}
+	bp = iip->ili_item.li_buf;
+	iip->ili_item.li_buf = NULL;
 
 	/*
 	 * Flush the inode and disassociate it from the transaction regardless
 	 * of whether the flush succeed or not. If we fail the flush, make sure
 	 * we still release the buffer reference we currently hold.
 	 */
-	error = libxfs_iflush_int(ip, bp);
+	error = libxfs_iflush_int(iip->ili_inode, bp);
 	bp->b_transp = NULL;	/* remove xact ptr */
 
 	if (error) {
 		fprintf(stderr, _("%s: warning - iflush_int failed (%d)\n"),
 			progname, error);
-		libxfs_buf_relse(bp);
 		goto free;
 	}
 
 	libxfs_buf_mark_dirty(bp);
-	libxfs_buf_relse(bp);
 free:
+	libxfs_buf_relse(bp);
+free_item:
 	xfs_inode_item_put(iip);
 }
 
@@ -874,7 +866,7 @@ trans_committed(
 		if (lip->li_type == XFS_LI_BUF)
 			buf_item_done((xfs_buf_log_item_t *)lip);
 		else if (lip->li_type == XFS_LI_INODE)
-			inode_item_done((xfs_inode_log_item_t *)lip);
+			inode_item_done((struct xfs_inode_log_item *)lip);
 		else {
 			fprintf(stderr, _("%s: unrecognised log item type\n"),
 				progname);
@@ -902,7 +894,7 @@ buf_item_unlock(
 
 static void
 inode_item_unlock(
-	xfs_inode_log_item_t	*iip)
+	struct xfs_inode_log_item	*iip)
 {
 	xfs_inode_item_put(iip);
 }
@@ -919,7 +911,7 @@ xfs_trans_free_items(
 		if (lip->li_type == XFS_LI_BUF)
 			buf_item_unlock((xfs_buf_log_item_t *)lip);
 		else if (lip->li_type == XFS_LI_INODE)
-			inode_item_unlock((xfs_inode_log_item_t *)lip);
+			inode_item_unlock((struct xfs_inode_log_item *)lip);
 		else {
 			fprintf(stderr, _("%s: unrecognised log item type\n"),
 				progname);
